<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2007 The Cacti Group                                      |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

function plugin_init_discovery() {
	global $plugin_hooks;
	$plugin_hooks['top_header_tabs']['discovery'] = 'discovery_show_tab';
	$plugin_hooks['top_graph_header_tabs']['discovery'] = 'discovery_show_tab';
	$plugin_hooks['config_arrays']['discovery'] = 'discovery_config_arrays';
	$plugin_hooks['draw_navigation_text']['discovery'] = 'discovery_draw_navigation_text';
	$plugin_hooks['config_settings']['discovery'] = 'discovery_config_settings';
	$plugin_hooks['poller_bottom']['discovery'] = 'discovery_poller_bottom';
	$plugin_hooks['utilities_action']['discovery'] = 'discovery_utilities_action';
	$plugin_hooks['utilities_list']['discovery'] = 'discovery_utilities_list';
}

function discovery_version () {
	return array( 	'name' 		=> 'discovery',
			'version' 	=> '0.8.2',
			'longname'	=> 'Network Discovery',
			'author'	=> 'Jimmy Conner',
			'homepage'	=> 'http://cactiusers.org',
			'email'		=> 'jimmy@sqmail.org',
			'url'		=> 'http://cactiusers.org/cacti/versions.php'
			);
}

function discovery_utilities_action ($action) {
	if ($action == 'discovery_clear') {
		mysql_query('DELETE FROM plugin_discover_hosts');

		include_once('./include/top_header.php');

		utilities();

		include_once('./include/bottom_footer.php');
	}
	return $action;
}

function discovery_utilities_list () {
	global $colors;

	html_header(array("Discovery Results"), 2);
	?>
	<tr bgcolor="#<?php print $colors["form_alternate1"];?>">
		<td class="textArea">
			<p><a href='utilities.php?action=discovery_clear'>Clear Discovery Results</a></p>
		</td>
		<td class="textArea">
			<p>This will clear the results from the discovery polling.</p>
		</td>
	</tr>
	<?php
}

function discovery_config_settings () {
	global $tabs, $settings, $discover_poller_frequencies;
	$tabs["misc"] = "Misc";

	if (isset($_SERVER['PHP_SELF']) && basename($_SERVER['PHP_SELF']) != 'settings.php')
		return;

	$temp = array(
		"discovery_header" => array(
			"friendly_name" => "Discover",
			"method" => "spacer",
			),
		"discovery_subnet" => array(
			"friendly_name" => "Subnet to scan",
			"description" => "This is the subnet we will scan.  (Use commas for multiple subnets.  ex: 192.168.100.*,192.168.0.0/24)",
			"method" => "textbox",
			"max_length" => 255,
			),
		"discovery_dns" => array(
			"friendly_name" => "DNS Server",
			"description" => "This is the DNS Server used to resolve names.  Leave blank to disable resolving.",
			"method" => "textbox",
			"max_length" => 255,
			),
		"discovery_protocol" => array(
			"friendly_name" => "Ping Method",
			"description" => "This is the type of protocol used by Ping to determine if the host is responding.",
			"method" => "drop_array",
			"array" => array(0 => 'UDP', 1 => 'TCP', 2 => 'ICMP'),
			"default" => 0
			),
		"discovery_readstrings" => array(
			"friendly_name" => "SNMP Communities",
			"description" => "Fill in the list of available SNMP Community Names to test for this device. Each Community Name must be separated by a colon ':'. These will be tested sequentially.",
			"method" => "textbox",
			"max_length" => 255,
			"default" => "public"
			),
		"discovery_collection_timing" => array(
			"friendly_name" => "Poller Frequency",
			"description" => "Choose how often to attempt to find devices on  your network.",
			"method" => "drop_array",
			"default" => "disabled",
			"array" => $discover_poller_frequencies,
			),
		"discovery_base_time" => array(
			"friendly_name" => "Start Time for Polling",
			"description" => "When would you like the first polling to take place.  All future polling times will be based upon this start time.  A good example would be 12:00AM.",
			"default" => "12:00am",
			"method" => "textbox",
			"max_length" => "10"
			),
		'discovery_query_rerun' => array(
			'friendly_name' => 'Rerun Data Queries',
			'description' => 'This option will rerun all data queries on current hosts, and will create graphs for all assigned graph templates and data queries.',
			'method' => 'checkbox',
			),
		'discovery_interface_up_only' => array(
			'friendly_name' => 'Create Graphs for Up Interfaces Only',
			'description' => 'This option will create graphs for interfaces that are showing as Up.',
			'method' => 'checkbox',
			),
	);
	if (isset($settings["misc"]))
		$settings["misc"] = array_merge($settings["misc"], $temp);
	else
		$settings["misc"]=$temp;
}

function discovery_show_tab () {
	global $config, $discovery_tab;
	include_once($config["library_path"] . "/database.php");
	include_once($config["base_path"] . "/plugins/discovery/config.php");
	if (!$discovery_tab)
		return;
	if (api_user_realm_auth('discover.php')) {
		print '<a href="' . $config['url_path'] . 'plugins/discovery/discover.php"><img src="' . $config['url_path'] . 'plugins/discovery/images/tab_discover.gif" alt="Discover" align="absmiddle" border="0"></a>';
	}
}

function discovery_config_arrays () {
	global $user_auth_realms, $user_auth_realm_filenames, $menu, $config, $discovery_tab, $discover_poller_frequencies;

	$user_auth_realms[34]='View Host Auto-Discovery';
	$user_auth_realm_filenames['discover.php'] = 34;
	$user_auth_realm_filenames['discover_template.php'] = 34;

	include_once($config["base_path"] . "/plugins/discovery/config.php");

	$menu["Templates"]['plugins/discovery/discover_template.php'] = "Discovery Templates";

	if (!$discovery_tab) {
		$temp = $menu["Utilities"]['logout.php'];
		unset($menu["Utilities"]['logout.php']);
		$menu["Utilities"]['plugins/discovery/discover.php'] = "Discovery";
		$menu["Utilities"]['logout.php'] = $temp;
	}

	$discover_poller_frequencies = array(
		"disabled" => "Disabled",
		"60" => "Every 1 Hour",
		"120" => "Every 2 Hours",
		"240" => "Every 4 Hours",
		"360" => "Every 6 Hours",
		"480" => "Every 8 Hours",
		"720" => "Every 12 Hours",
		"1440" => "Every Day",
		"10080" => "Every Week",
		"20160" => "Every 2 Weeks",
		"40320" => "Every 4 Weeks"
		);

}

function discovery_draw_navigation_text ($nav) {
	$nav["discover.php:"] = array("title" => "Discover", "mapping" => "index.php:", "url" => "discover.php", "level" => "1");
	$nav["discover_template.php:"] = array("title" => "Discover Templates", "mapping" => "index.php:", "url" => "discover_template.php", "level" => "1");
	$nav["discover_template.php:edit"] = array("title" => "Discover Templates", "mapping" => "index.php:", "url" => "discover_template.php", "level" => "1");
	$nav["discover_template.php:actions"] = array("title" => "Discover Templates", "mapping" => "index.php:", "url" => "discover_template.php", "level" => "1");
	$nav["utilities.php:discovery_clear"] = array("title" => "Clear Discover Results", "mapping" => "index.php:,utilities.php:", "url" => "discover.php", "level" => "1");
	return $nav;
}

function discovery_setup_table () {
	global $config, $database_default;
	include_once($config["library_path"] . "/database.php");
	$sql = "show tables from `" . $database_default . "`";

	$result = db_fetch_assoc($sql) or die (mysql_error());

	$tables = array();
	$sql = array();

	foreach($result as $index => $arr) {
		foreach ($arr as $t) {
			$tables[] = $t;
		}
	}

	if (!in_array('plugin_discover_hosts', $tables)) {
		$sql[] = "CREATE TABLE plugin_discover_hosts (
				  hostname varchar(100) NOT NULL default '',
				  ip varchar(17) NOT NULL default '',
				  hash varchar(12) NOT NULL default '',
				  community varchar(100) NOT NULL default '',
				  sysName varchar(100) NOT NULL default '',
				  sysLocation varchar(255) NOT NULL default '',
				  sysContact varchar(255) NOT NULL default '',
				  sysDescr varchar(255) NOT NULL default '',
				  sysUptime int(32) NOT NULL default '0',
				  os varchar(64) NOT NULL default '',
				  snmp tinyint(4) NOT NULL default '0',
				  known tinyint(4) NOT NULL default '0',
				  up tinyint(4) NOT NULL default '0',
				  time int(11) NOT NULL default '0',
				  PRIMARY KEY  (ip)
				) TYPE=MyISAM;";
		$sql[] = "INSERT INTO user_auth_realm VALUES (34, 1);";
	}

	if (in_array('plugin_discover_os', $tables)) {
		$sql[] = "DROP TABLE plugin_discover_os";
	}

	if (!in_array('plugin_discover_template', $tables)) {
		$sql[] = "CREATE TABLE plugin_discover_template (
				  id int(8) NOT NULL auto_increment,
				  host_template int(8) NOT NULL default '0',
				  tree int(12) NOT NULL default '0',
				  snmp_version tinyint(3) NOT NULL default '0',
				  sysdescr varchar(255) NOT NULL default '',
				  PRIMARY KEY  (id)
				) TYPE=MyISAM;";
	}

	if (!empty($sql)) {
		for ($a = 0; $a < count($sql); $a++) {
			$result = mysql_query($sql[$a]);
		}
	}
}

function discovery_poller_bottom () {
	global $config;

	include_once($config["library_path"] . "/database.php");

	if (read_config_option("discovery_collection_timing") == "disabled")
		return;

	$t = read_config_option("discovery_last_poll");

	/* Check for the polling interval, only valid with the Multipoller patch */
	$poller_interval = read_config_option("poller_interval");
	if (!isset($poller_interval)) {
		$poller_interval = 300;
	}

	if ($t != '' && (time() - $t < $poller_interval))
		return;

	discovery_setup_table();

	$command_string = trim(read_config_option("path_php_binary"));

	// If its not set, just assume its in the path
	if (trim($command_string) == '')
		$command_string = "php";
	$extra_args = ' -q ' . $config['base_path'] . '/plugins/discovery/findhosts.php';

	exec_background($command_string, $extra_args);

	if ($t == "")
		$sql = "insert into settings values ('discovery_last_poll','" . time() . "')";
	else
		$sql = "update settings set value = '" . time() . "' where name = 'discovery_last_poll'";
	$result = mysql_query($sql) or die (mysql_error());
}

?>